@@ -8,23 +8,54 @@ Core components for emoji.
 
 import re
 import unicodedata
-from typing import Iterator
+import sys
+from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union
+
+if sys.version_info < (3, 9):
+    from typing_extensions import Literal, Match, TypedDict  # type: ignore
+else:
+    from typing import Literal, Match, TypedDict
 
 from emoji import unicode_codes
-from emoji.tokenizer import Token, EmojiMatch, EmojiMatchZWJ, EmojiMatchZWJNonRGI, tokenize, filter_tokens
+from emoji.tokenizer import (
+    Token,
+    EmojiMatch,
+    EmojiMatchZWJ,
+    EmojiMatchZWJNonRGI,
+    tokenize,
+    filter_tokens,
+)
 
 __all__ = [
-    'emojize', 'demojize', 'analyze', 'config',
-    'emoji_list', 'distinct_emoji_list', 'emoji_count',
-    'replace_emoji', 'is_emoji', 'purely_emoji', 'version',
-    'Token', 'EmojiMatch', 'EmojiMatchZWJ', 'EmojiMatchZWJNonRGI',
+    'emojize',
+    'demojize',
+    'analyze',
+    'config',
+    'emoji_list',
+    'distinct_emoji_list',
+    'emoji_count',
+    'replace_emoji',
+    'is_emoji',
+    'purely_emoji',
+    'version',
+    'Token',
+    'EmojiMatch',
+    'EmojiMatchZWJ',
+    'EmojiMatchZWJNonRGI',
 ]
 
 _DEFAULT_DELIMITER = ':'
-_EMOJI_NAME_PATTERN = '\\w\\-&.’”“()!#*+,/«»\u0300\u0301\u0302\u0303\u0308\u030a\u0327\u064b\u064e\u064f\u0650\u0653\u0654\u3099\u30fb\u309a'
+# In Arabic language, the unicode character "\u0655" should be kept so we add it to the pattern below
+_EMOJI_NAME_PATTERN = '\\w\\-&.’”“()!#*+,/«»\u0300\u0301\u0302\u0303\u0306\u0308\u030a\u0327\u064b\u064e\u064f\u0650\u0653\u0654\u3099\u30fb\u309a\u0655'
 
 
-class config():
+class _EmojiListReturn(TypedDict):
+    emoji: str
+    match_start: int
+    match_end: int
+
+
+class config:
     """Module-wide configuration"""
 
     demojize_keep_zwj = True
@@ -60,15 +91,42 @@ class config():
     See :attr:`config.demojize_keep_zwj` for more information.
     """
 
+    @staticmethod
+    def load_language(language: Union[List[str], str, None] = None):
+        """Load one or multiple languages into memory.
+        If no language is specified, all languages will be loaded.
+
+        This makes language data accessible in the :data:`EMOJI_DATA` dict.
+        For example to access a French emoji name, first load French with
+
+         ``emoji.config.load_language('fr')``
+
+        and then access it with
+
+         ``emoji.EMOJI_DATA['🏄']['fr']``
+
+        Available languages are listed in :data:`LANGUAGES`"""
+
+        languages = (
+            [language]
+            if isinstance(language, str)
+            else language
+            if language
+            else unicode_codes.LANGUAGES
+        )
+
+        for lang in languages:
+            unicode_codes.load_from_json(lang)
+
 
 def emojize(
-        string,
-        delimiters=(_DEFAULT_DELIMITER, _DEFAULT_DELIMITER),
-        variant=None,
-        language='en',
-        version=None,
-        handle_version=None
-):
+    string: str,
+    delimiters: Tuple[str, str] = (_DEFAULT_DELIMITER, _DEFAULT_DELIMITER),
+    variant: Optional[Literal['text_type', 'emoji_type']] = None,
+    language: str = 'en',
+    version: Optional[float] = None,
+    handle_version: Optional[Union[str, Callable[[str, Dict[str, str]], str]]] = None,
+) -> str:
     """
     Replace emoji names in a string with Unicode codes.
         >>> import emoji
@@ -112,20 +170,22 @@ def emojize(
 
     """
 
-    if language == 'alias':
-        language_pack = unicode_codes.get_aliases_unicode_dict()
-    else:
-        language_pack = unicode_codes.get_emoji_unicode_dict(language)
+    unicode_codes.load_from_json(language)
 
-    pattern = re.compile('(%s[%s]+%s)' %
-                         (re.escape(delimiters[0]), _EMOJI_NAME_PATTERN, re.escape(delimiters[1])))
+    pattern = re.compile(
+        '(%s[%s]+%s)'
+        % (re.escape(delimiters[0]), _EMOJI_NAME_PATTERN, re.escape(delimiters[1]))
+    )
+
+    def replace(match: Match[str]) -> str:
+        name = match.group(1)[len(delimiters[0]) : -len(delimiters[1])]
+        emj = unicode_codes.get_emoji_by_name(
+            _DEFAULT_DELIMITER
+            + unicodedata.normalize('NFKC', name)
+            + _DEFAULT_DELIMITER,
+            language,
+        )
 
-    def replace(match):
-        name = match.group(1)[len(delimiters[0]):-len(delimiters[1])]
-        emj = language_pack.get(
-            _DEFAULT_DELIMITER +
-            unicodedata.normalize('NFKC', name) +
-            _DEFAULT_DELIMITER)
         if emj is None:
             return match.group(1)
 
@@ -144,24 +204,27 @@ def emojize(
         if variant is None or 'variant' not in unicode_codes.EMOJI_DATA[emj]:
             return emj
 
-        if emj[-1] == '\uFE0E' or emj[-1] == '\uFE0F':
+        if emj[-1] == '\ufe0e' or emj[-1] == '\ufe0f':
             # Remove an existing variant
             emj = emj[0:-1]
-        if variant == "text_type":
-            return emj + '\uFE0E'
-        elif variant == "emoji_type":
-            return emj + '\uFE0F'
+        if variant == 'text_type':
+            return emj + '\ufe0e'
+        elif variant == 'emoji_type':
+            return emj + '\ufe0f'
         else:
             raise ValueError(
-                "Parameter 'variant' must be either None, 'text_type' or 'emoji_type'")
+                "Parameter 'variant' must be either None, 'text_type' or 'emoji_type'"
+            )
 
     return pattern.sub(replace, string)
 
 
-def analyze(string: str, non_emoji: bool = False, join_emoji: bool = True) -> Iterator[Token]:
+def analyze(
+    string: str, non_emoji: bool = False, join_emoji: bool = True
+) -> Iterator[Token]:
     """
     Find unicode emoji in a string. Yield each emoji as a named tuple
-    :class:`Token` ``(chars, EmojiMatch)`` or `:class:`Token` ``(chars, EmojiMatchZWJNonRGI)``.
+    :class:`Token` ``(chars, EmojiMatch)`` or :class:`Token` ``(chars, EmojiMatchZWJNonRGI)``.
     If ``non_emoji`` is True, also yield all other characters as
     :class:`Token` ``(char, char)`` .
 
@@ -172,16 +235,17 @@ def analyze(string: str, non_emoji: bool = False, join_emoji: bool = True) -> It
     """
 
     return filter_tokens(
-        tokenize(string, keep_zwj=True), emoji_only=not non_emoji, join_emoji=join_emoji)
+        tokenize(string, keep_zwj=True), emoji_only=not non_emoji, join_emoji=join_emoji
+    )
 
 
 def demojize(
-        string,
-        delimiters=(_DEFAULT_DELIMITER, _DEFAULT_DELIMITER),
-        language='en',
-        version=None,
-        handle_version=None
-):
+    string: str,
+    delimiters: Tuple[str, str] = (_DEFAULT_DELIMITER, _DEFAULT_DELIMITER),
+    language: str = 'en',
+    version: Optional[float] = None,
+    handle_version: Optional[Union[str, Callable[[str, Dict[str, str]], str]]] = None,
+) -> str:
     """
     Replace Unicode emoji in a string with emoji shortcodes. Useful for storage.
         >>> import emoji
@@ -223,7 +287,10 @@ def demojize(
     else:
         _use_aliases = False
 
-    def handle(emoji_match):
+    unicode_codes.load_from_json(language)
+
+    def handle(emoji_match: EmojiMatch) -> str:
+        assert emoji_match.data is not None
         if version is not None and emoji_match.data['E'] > version:
             if callable(handle_version):
                 return handle_version(emoji_match.emoji, emoji_match.data_copy())
@@ -233,7 +300,9 @@ def demojize(
                 return ''
         elif language in emoji_match.data:
             if _use_aliases and 'alias' in emoji_match.data:
-                return delimiters[0] + emoji_match.data['alias'][0][1:-1] + delimiters[1]
+                return (
+                    delimiters[0] + emoji_match.data['alias'][0][1:-1] + delimiters[1]
+                )
             else:
                 return delimiters[0] + emoji_match.data[language][1:-1] + delimiters[1]
         else:
@@ -241,11 +310,17 @@ def demojize(
             return emoji_match.emoji
 
     matches = tokenize(string, keep_zwj=config.demojize_keep_zwj)
-    return "".join(str(handle(token.value)) if isinstance(
-        token.value, EmojiMatch) else token.value for token in matches)
+    return ''.join(
+        str(handle(token.value)) if isinstance(token.value, EmojiMatch) else token.value
+        for token in matches
+    )
 
 
-def replace_emoji(string, replace='', version=-1):
+def replace_emoji(
+    string: str,
+    replace: Union[str, Callable[[str, Dict[str, str]], str]] = '',
+    version: float = -1,
+) -> str:
     """
     Replace Unicode emoji in a customizable string.
 
@@ -258,8 +333,9 @@ def replace_emoji(string, replace='', version=-1):
         only emoji above this version will be replaced.
     """
 
-    def handle(emoji_match):
+    def handle(emoji_match: EmojiMatch) -> str:
         if version > -1:
+            assert emoji_match.data is not None
             if emoji_match.data['E'] > version:
                 if callable(replace):
                     return replace(emoji_match.emoji, emoji_match.data_copy())
@@ -267,41 +343,44 @@ def replace_emoji(string, replace='', version=-1):
                     return str(replace)
         elif callable(replace):
             return replace(emoji_match.emoji, emoji_match.data_copy())
-        elif replace is not None:
+        elif replace is not None:  # type: ignore
             return replace
         return emoji_match.emoji
 
     matches = tokenize(string, keep_zwj=config.replace_emoji_keep_zwj)
     if config.replace_emoji_keep_zwj:
-        matches = filter_tokens(
-            matches, emoji_only=False, join_emoji=True)
-    return "".join(str(handle(m.value)) if isinstance(
-        m.value, EmojiMatch) else m.value for m in matches)
+        matches = filter_tokens(matches, emoji_only=False, join_emoji=True)
+    return ''.join(
+        str(handle(m.value)) if isinstance(m.value, EmojiMatch) else m.value
+        for m in matches
+    )
 
 
-def emoji_list(string):
+def emoji_list(string: str) -> List[_EmojiListReturn]:
     """
     Returns the location and emoji in list of dict format.
         >>> emoji.emoji_list("Hi, I am fine. 😁")
         [{'match_start': 15, 'match_end': 16, 'emoji': '😁'}]
     """
 
-    return [{
-        'match_start': m.value.start,
-        'match_end': m.value.end,
-        'emoji': m.value.emoji,
-    } for m in tokenize(string, keep_zwj=False) if isinstance(m.value, EmojiMatch)]
+    return [
+        {
+            'match_start': m.value.start,
+            'match_end': m.value.end,
+            'emoji': m.value.emoji,
+        }
+        for m in tokenize(string, keep_zwj=False)
+        if isinstance(m.value, EmojiMatch)
+    ]
 
 
-def distinct_emoji_list(string):
+def distinct_emoji_list(string: str) -> List[str]:
     """Returns distinct list of emojis from the string."""
-    distinct_list = list(
-        {e['emoji'] for e in emoji_list(string)}
-    )
+    distinct_list = list({e['emoji'] for e in emoji_list(string)})
     return distinct_list
 
 
-def emoji_count(string, unique=False):
+def emoji_count(string: str, unique: bool = False) -> int:
     """
     Returns the count of emojis in a string.
 
@@ -312,7 +391,7 @@ def emoji_count(string, unique=False):
     return len(emoji_list(string))
 
 
-def is_emoji(string):
+def is_emoji(string: str) -> bool:
     """
     Returns True if the string is a single emoji, and it is "recommended for
     general interchange" by Unicode.org.
@@ -329,7 +408,7 @@ def purely_emoji(string: str) -> bool:
     return all(isinstance(m.value, EmojiMatch) for m in analyze(string, non_emoji=True))
 
 
-def version(string):
+def version(string: str) -> float:
     """
     Returns the Emoji Version of the emoji.
 
@@ -346,27 +425,27 @@ def version(string):
     if string in unicode_codes.EMOJI_DATA:
         return unicode_codes.EMOJI_DATA[string]['E']
 
-    language_pack = unicode_codes.get_emoji_unicode_dict('en')
-    if string in language_pack:
-        emj_code = language_pack[string]
-        if emj_code in unicode_codes.EMOJI_DATA:
-            return unicode_codes.EMOJI_DATA[emj_code]['E']
+    # Try name lookup
+    emj_code = unicode_codes.get_emoji_by_name(string, 'en')
+    if emj_code and emj_code in unicode_codes.EMOJI_DATA:
+        return unicode_codes.EMOJI_DATA[emj_code]['E']
 
     # Try to find first emoji in string
-    version = []
+    version: List[float] = []
 
-    def f(e, emoji_data):
+    def f(e: str, emoji_data: Dict[str, Any]) -> str:
         version.append(emoji_data['E'])
         return ''
+
     replace_emoji(string, replace=f, version=-1)
     if version:
         return version[0]
     emojize(string, language='alias', version=-1, handle_version=f)
     if version:
         return version[0]
-    for lang_code in unicode_codes._EMOJI_UNICODE:
+    for lang_code in unicode_codes.LANGUAGES:
         emojize(string, language=lang_code, version=-1, handle_version=f)
         if version:
             return version[0]
 
-    raise ValueError("No emoji found in string")
+    raise ValueError('No emoji found in string')
